import type { DocumentNode, ExecutionResult, GraphQLSchema } from 'graphql';
import { Anyware } from '../../lib/anyware/__.js';
import { type StandardScalarVariables } from '../../lib/graphql.js';
import type { Schema } from '../1_Schema/__.js';
import type { GraphQLObjectSelection } from '../3_SelectionSet/encode.js';
import type { ContextInterfaceRaw, ContextInterfaceTyped, InterfaceRaw, InterfaceTyped, TransportHttp, TransportMemory } from './types.js';
type InterfaceInput<TypedProperties = {}, RawProperties = {}> = ({
    interface: InterfaceTyped;
    context: ContextInterfaceTyped;
    rootTypeName: Schema.RootTypeName;
} & TypedProperties) | ({
    interface: InterfaceRaw;
    context: ContextInterfaceRaw;
} & RawProperties);
type TransportInput<HttpProperties = {}, MemoryProperties = {}> = ({
    transport: TransportHttp;
} & HttpProperties) | ({
    transport: TransportMemory;
} & MemoryProperties);
export declare const hookNamesOrderedBySequence: readonly ["encode", "pack", "exchange", "unpack", "decode"];
export type HookSequence = typeof hookNamesOrderedBySequence;
export type HookDefEncode = {
    input: InterfaceInput<{
        selection: GraphQLObjectSelection;
    }, {
        document: string | DocumentNode;
        variables?: StandardScalarVariables;
        operationName?: string;
    }> & TransportInput<{
        schema: string | URL;
    }, {
        schema: GraphQLSchema;
    }>;
    slots: {
        /**
         * Create the value that will be used as the HTTP body for the sent GraphQL request.
         */
        body: (input: {
            query: string;
            variables?: StandardScalarVariables;
            operationName?: string;
        }) => BodyInit;
    };
};
export type HookDefPack = {
    input: InterfaceInput & TransportInput<{
        url: string | URL;
        headers?: HeadersInit;
        body: BodyInit;
    }, {
        schema: GraphQLSchema;
        query: string;
        variables?: StandardScalarVariables;
        operationName?: string;
    }>;
};
type RequestInput = {
    url: string | URL;
    method: 'get' | 'post' | 'put' | 'delete' | 'patch' | 'head' | 'options' | 'trace' | 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH' | 'HEAD' | 'OPTIONS' | 'TRACE';
    headers?: HeadersInit;
    body: BodyInit;
};
export type HookDefExchange = {
    slots: {
        fetch: typeof fetch;
    };
    input: InterfaceInput & TransportInput<{
        request: RequestInput;
    }, {
        schema: GraphQLSchema;
        query: string | DocumentNode;
        variables?: StandardScalarVariables;
        operationName?: string;
    }>;
};
export type HookDefUnpack = {
    input: InterfaceInput & TransportInput<{
        response: Response;
    }, {
        result: ExecutionResult;
    }>;
};
export type HookDefDecode = {
    input: {
        result: ExecutionResult;
    } & InterfaceInput & TransportInput<{
        response: Response;
    }>;
};
export type HookMap = {
    encode: HookDefEncode;
    pack: HookDefPack;
    exchange: HookDefExchange;
    unpack: HookDefUnpack;
    decode: HookDefDecode;
};
export declare const anyware: Anyware.Builder<Anyware.Core<readonly ["encode", "pack", "exchange", "unpack", "decode"], HookMap, ExecutionResult<import("graphql/jsutils/ObjMap.js").ObjMap<unknown>, import("graphql/jsutils/ObjMap.js").ObjMap<unknown>>>>;
export type Core = (typeof anyware)['core'];
export {};
//# sourceMappingURL=core.d.ts.map