import type { ExecutionResult as GraphQLExecutionResult } from 'graphql';
import type { Simplify } from 'type-fest';
import type { GraphQLExecutionResultError } from '../../../lib/graphql.js';
import type { ConfigManager, StringKeyof } from '../../../lib/prelude.js';
import type { Schema } from '../../1_Schema/__.js';
import type { GlobalRegistry } from '../../2_generator/globalRegistry.js';
import type { SelectionSet } from '../../3_SelectionSet/__.js';
import type { Transport } from '../../5_core/types.js';
export type OutputChannel = 'throw' | 'return';
export type OutputChannelConfig = 'throw' | 'return' | 'default';
export type ErrorCategory = 'execution' | 'other' | 'schema';
export declare const readConfigErrorCategoryOutputChannel: (config: Config, errorCategory: ErrorCategory) => OutputChannel | false;
export declare const traditionalGraphqlOutput: OutputConfig;
export declare const traditionalGraphqlOutputThrowing: OutputConfig;
export declare const isContextConfigTraditionalGraphQLOutput: (config: Config) => boolean;
export type OutputConfig = {
    defaults: {
        errorChannel: OutputChannel;
    };
    envelope: {
        enabled: boolean;
        errors: {
            execution: boolean;
            other: boolean;
            schema: boolean;
        };
    };
    errors: {
        execution: OutputChannelConfig;
        other: OutputChannelConfig;
        schema: false | OutputChannelConfig;
    };
};
export declare const outputConfigDefault: OutputConfigDefault;
export type OutputConfigDefault = {
    defaults: {
        errorChannel: 'throw';
    };
    envelope: {
        enabled: false;
        errors: {
            execution: true;
            other: false;
            schema: false;
        };
    };
    errors: {
        execution: 'default';
        other: 'default';
        schema: false;
    };
};
export type Config = {
    output: OutputConfig;
    transport: Transport;
};
export type ResolveOutputReturnRootType<$Config extends Config, $Index extends Schema.Index, $Data> = IfConfiguredGetOutputErrorReturns<$Config> | ($Config['output']['envelope']['enabled'] extends true ? Envelope<$Config, IfConfiguredStripSchemaErrorsFromDataRootType<$Config, $Index, $Data>> : Simplify<IfConfiguredStripSchemaErrorsFromDataRootType<$Config, $Index, $Data>>);
export type ResolveOutputReturnRootField<$Config extends Config, $Index extends Schema.Index, $Data, $DataRaw = undefined> = IfConfiguredGetOutputErrorReturns<$Config> | ($Config['output']['envelope']['enabled'] extends true ? Envelope<$Config, $DataRaw extends undefined ? IfConfiguredStripSchemaErrorsFromDataRootField<$Config, $Index, $Data> : IfConfiguredStripSchemaErrorsFromDataRootType<$Config, $Index, $DataRaw>> : Simplify<IfConfiguredStripSchemaErrorsFromDataRootField<$Config, $Index, $Data>>);
export type Envelope<$Config extends Config, $Data = unknown> = Simplify<$Config['transport'] extends 'http' ? EnvelopeTransportHttp<$Data> : EnvelopeTransportMemory<$Data>>;
export type EnvelopeTransportHttp<$Data> = GraphQLExecutionResult<$Data> & {
    response: Response;
};
export type EnvelopeTransportMemory<$Data> = GraphQLExecutionResult<$Data>;
type ConfigResolveOutputErrorChannel<$Config extends Config, $Channel extends OutputChannelConfig | false> = $Channel extends 'default' ? $Config['output']['defaults']['errorChannel'] : $Channel extends false ? false : $Channel;
type ConfigGetOutputEnvelopeErrorChannel<$Config extends Config, $ErrorCategory extends ErrorCategory> = $Config['output']['envelope']['errors'][$ErrorCategory] extends true ? false : ConfigResolveOutputErrorChannel<$Config, $Config['output']['errors'][$ErrorCategory]>;
type ConfigGetOutputError<$Config extends Config, $ErrorCategory extends ErrorCategory> = $Config['output']['envelope']['enabled'] extends true ? ConfigGetOutputEnvelopeErrorChannel<$Config, $ErrorCategory> : ConfigResolveOutputErrorChannel<$Config, $Config['output']['errors'][$ErrorCategory]>;
type IfConfiguredGetOutputErrorReturns<$Config extends Config> = (ConfigGetOutputError<$Config, 'execution'> extends 'return' ? GraphQLExecutionResultError : never) | (ConfigGetOutputError<$Config, 'other'> extends 'return' ? Error : never) | (ConfigGetOutputError<$Config, 'schema'> extends 'return' ? Error : never);
type IfConfiguredStripSchemaErrorsFromDataRootType<$Config extends Config, $Index extends Schema.Index, $Data> = {
    [$RootFieldName in keyof $Data]: IfConfiguredStripSchemaErrorsFromDataRootField<$Config, $Index, $Data[$RootFieldName]>;
};
type IfConfiguredStripSchemaErrorsFromDataRootField<$Config extends Config, $Index extends Schema.Index, $Data> = $Config['output']['errors']['schema'] extends false ? $Data : ExcludeSchemaErrors<$Index, $Data>;
export type ExcludeSchemaErrors<$Index extends Schema.Index, $Data> = Exclude<$Data, $Index['error']['objectsTypename'][keyof $Index['error']['objectsTypename']]>;
export type OrThrowifyConfig<$Config extends Config> = ConfigManager.Set<$Config, ['output', 'errors'], {
    other: 'throw';
    execution: 'throw';
    schema: 'throw';
}>;
/**
 * We inject __typename select when:
 * 1. using schema errors
 * 2. using return mode dataSuccess
 */
type TypenameSelection = {
    __typename: true;
};
export type CreateSelectionTypename<$Config extends Config, $Index extends Schema.Index> = IsNeedSelectionTypename<$Config, $Index> extends true ? TypenameSelection : {};
export type IsNeedSelectionTypename<$Config extends Config, $Index extends Schema.Index> = ConfigGetOutputError<$Config, 'schema'> extends 'throw' ? GlobalRegistry.HasSchemaErrorsViaName<$Index['name']> extends true ? true : false : false;
export type AugmentRootTypeSelectionWithTypename<$Config extends Config, $Index extends Schema.Index, $RootTypeName extends Schema.RootTypeName, $Selection extends object> = IsNeedSelectionTypename<$Config, $Index> extends true ? {
    [$Key in StringKeyof<$Selection>]: $Selection[$Key] & (IsRootFieldNameAResultField<$Index, $RootTypeName, $Key> extends true ? TypenameSelection : {});
} : $Selection;
type IsRootFieldNameAResultField<$Index extends Schema.Index, $RootTypeName extends Schema.RootTypeName, $FieldName extends string> = SelectionSet.AliasNameOrigin<$FieldName> extends keyof $Index['error']['rootResultFields'][$RootTypeName] ? true : false;
export {};
//# sourceMappingURL=Config.d.ts.map