import { type ExecutionResult, GraphQLSchema } from 'graphql';
import type { Anyware } from '../../lib/anyware/__.js';
import type { SomeExecutionResultWithoutErrors } from '../../lib/graphql.js';
import type { Exact } from '../../lib/prelude.js';
import type { URLInput } from '../0_functions/request.js';
import type { BaseInput } from '../0_functions/types.js';
import { Schema } from '../1_Schema/__.js';
import type { GlobalRegistry } from '../2_generator/globalRegistry.js';
import { Core } from '../5_core/__.js';
import type { DocumentFn } from './document.js';
import type { GetRootTypeMethods } from './RootTypeMethods.js';
import type { Envelope } from './Settings/Config.js';
import { type Config } from './Settings/Config.js';
import { type Input, type InputPrefilled, type InputToConfig } from './Settings/Input.js';
export type SchemaInput = URLInput | GraphQLSchema;
export type SelectionSetOrIndicator = 0 | 1 | boolean | object;
export type SelectionSetOrArgs = object;
export interface Context {
    retry: undefined | Anyware.Extension2<Core.Core, {
        retrying: true;
    }>;
    extensions: Extension[];
    config: Config;
}
export type TypedContext = Context & {
    schemaIndex: Schema.Index;
};
interface RawInput extends BaseInput {
}
export type ClientRaw<$Config extends Config> = {
    raw(input: RawInput): Promise<Envelope<$Config>>;
    raw(document: RawInput['document'], options?: Omit<RawInput, 'document'>): Promise<ExecutionResult>;
    rawOrThrow(input: RawInput): Promise<SomeExecutionResultWithoutErrors>;
    rawOrThrow(document: RawInput['document'], options?: Omit<RawInput, 'document'>): Promise<SomeExecutionResultWithoutErrors>;
};
export type Extension = {
    name: string;
    anyware?: Anyware.Extension2<Core.Core>;
};
export type Client<$Index extends Schema.Index | null, $Config extends Config> = ClientRaw<$Config> & ($Index extends Schema.Index ? ClientTyped<$Index, $Config> : {}) & {
    use: (extension: Extension | Anyware.Extension2<Core.Core>) => Client<$Index, $Config>;
    retry: (extension: Anyware.Extension2<Core.Core, {
        retrying: true;
    }>) => Client<$Index, $Config>;
};
export type ClientTyped<$Index extends Schema.Index, $Config extends Config> = {
    document: DocumentFn<$Config, $Index>;
} & GetRootTypeMethods<$Config, $Index>;
export type CreatePrefilled = <$Name extends GlobalRegistry.SchemaNames>(name: $Name, schemaIndex: Schema.Index) => <$Input extends InputPrefilled<GlobalRegistry.Schemas[$Name]>>(input: Exact<$Input, InputPrefilled<GlobalRegistry.Schemas[$Name]>>) => Client<GlobalRegistry.GetSchemaIndexOrDefault<$Name>, InputToConfig<$Input>>;
export declare const createPrefilled: CreatePrefilled;
type Create = <$Input extends Input<GlobalRegistry.SchemaList>>(input: $Input) => Client<$Input['schemaIndex'] extends Schema.Index ? GlobalRegistry.GetSchemaIndexOrDefault<$Input['name']> : null, InputToConfig<$Input>>;
export declare const create: Create;
interface CreateState {
    retry?: Anyware.Extension2<Core.Core, {
        retrying: true;
    }>;
    extensions: Extension[];
}
export declare const createInternal: (input_: Input<any>, state: CreateState) => any;
export {};
//# sourceMappingURL=client.d.ts.map